\ ----------------------------------------------------------------------
\ LANGUAGE    : AVR ByteForth vsn 2.00
\ PROJECT     : PBM met timer 0
\ DESCRIPTION : Puls breedte modulator met timer-0
\ CATEGORY    : Applicatie, afmeting: 208 bytes.
\ AUTHOR      : Willem Ouwerkerk, April 8, 2002
\ LAST CHANGE : Willem Ouwerkerk, Juli 22, 2002
\ ----------------------------------------------------------------------

90S2313                                 \ Selecteer target
NEEDS TARGET.FRT                        \ Laad cpu specifiek deel

PORTB 0 BIT-SFR UITGANG                 \ PB.0 = gekozen PBM uitgang
TCCR0 SFR TIMER-CONTROLE                \ Timer besturing
TCNT0 SFR TIMER                         \ Timer register
TIMSK SFR TIMER-INTRPT                  \ Timer interrupt activering

#50 CONSTANT CYCLUS                     \ PBM periodeduur
REGISTER AANTIJD                        \ PBM tijd dat uitg. hoog is
REGISTER UITTIJD                        \ PBM tijd dat uitg. laag is.

: SETUP-PBM         ( -- )
    -1 TO TIMER                         \ Dummy timer inhoud
    2 TO TIMER-INTRPT                   \ Timer-0 interrupt aan
    2 TO TIMER-CONTROLE                 \ Prescaler = 8
    -1 SETDIR UITGANG                   \ PB.0 is uitgang
    ;

\ Ongeveer 28 cycles van elke 400 wordt voor PBM gebruikt ongev. 8%
\ De frequentie is 4.000.000/8 ~ 500.000/50 ~ 10 kHz
\ Deze methode regelt van ~ 2% tot 98 %
CODE PBM            ( -- )
    ADR UITGANG SBIS,                   \ Sla AHEAD over als uitg. hoog is
    AHEAD,                              \ Ga door na then
        ADR TIMER ADR UITTIJD OUT,      \ Zet duur lage periode
        ADR UITGANG CBI,                \ Uitgang laag
        RETI,                           \ Klaar
    THEN,
    ADR TIMER ADR AANTIJD OUT,          \ Zet hoge periode
    ADR UITGANG SBI,                    \ Uitgang hoog
    RETI,                               \ Klaar
END-CODE  T0-OVERFLOW

: SNELHEID          ( u -- )
    1 UMAX CYCLUS 1- UMIN               \ Houdt u tussen 1 en cyclus-1
    DUP NEGATE TO AANTIJD               \ Zet hoge periode
    CYCLUS - TO UITTIJD                 \ Zet lage periode
    ;

\ Toon PBM op PB.0
: PBM-DEMO          ( -- )
    SETUP-BYTEFORTH
    CLEAR UITGANG   255 MS              \ Toon opstarten
    SET UITGANG     255 MS
    INTERRUPT-ON
    SETUP-PBM
    BEGIN
        CYCLUS FOR                      \ Doorloop hele cyclus
            I' SNELHEID  100 MS
        NEXT
        CYCLUS SNELHEID                 \ Een sec. 100%
        4 FOR  250 MS  NEXT
        0 SNELHEID                      \ Een sec. 0%
        4 FOR  250 MS  NEXT
    AGAIN
    ;  MAIN

                              ( Einde )
